@@ -26,8 +26,7 @@ import uuid
 
 import psutil
 
-from .exceptions import BreakException, ContinueException, ReturnException, WrappedException
-from .scope import Scope
+from .exceptions import BreakException, ContinueException, ReturnException, WrappedException, StopAsyncIterationWithValue
 
 
 class ASTInterpreter:
@@ -398,7 +397,10 @@ class ASTInterpreter:
         method_name: str = "visit_" + node.__class__.__name__
         method = getattr(self, method_name, self.generic_visit)
         self.env_stack[0]["logger"].debug("Visiting %s at line %s" % (method_name, getattr(node, 'lineno', 'unknown')))
+        if method_name == "visit_Call":
+            self.env_stack[0]['logger'].debug(f"Debug: Node.func type for Call: {type(node.func).__name__ if node.func else 'None'}")
         
+        self.env_stack[0]["logger"].debug(f"Entering visit for {method_name}, is_await_context: {is_await_context}, wrap_exceptions: {wrap_exceptions}")
         try:
             if self.sync_mode:
                 from .execution_utils import sync_call
@@ -411,12 +413,40 @@ class ASTInterpreter:
                 result = await method(node, is_await_context, wrap_exceptions)
             else:
                 result = await method(node, wrap_exceptions=wrap_exceptions)
+            self.env_stack[0]["logger"].debug(f"Visit {method_name} completed, result: {result}, type: {type(result).__name__}")
             self.recursion_depth -= 1
             return result
-        except (ReturnException, BreakException, ContinueException):
+        except StopIteration as e:
+            self.env_stack[0]["logger"].debug(f"Caught StopIteration with value: {getattr(e, 'value', None)}")
+            self.recursion_depth -= 1
+            raise e
+        except StopAsyncIteration as e:
+            self.env_stack[0]["logger"].debug(f"Caught StopAsyncIteration with value: {getattr(e, 'value', None)}")
+            self.recursion_depth -= 1
+            raise e
+        except RuntimeError as e:
+            self.env_stack[0]["logger"].debug(f"Caught RuntimeError: {str(e)}, is_await_context: {is_await_context}")
+            self.recursion_depth -= 1
+            if str(e) == "coroutine raised StopIteration":
+                # Extract original StopIteration value from exception cause
+                orig = e.__cause__ if hasattr(e, '__cause__') else None
+                value = getattr(orig, 'value', None) if orig is not None else None
+                self.env_stack[0]["logger"].debug(f"Reconstructing StopIteration with value: {value}")
+                raise StopIteration(value)
+            if not wrap_exceptions:
+                raise
+            lineno = getattr(node, "lineno", None) or 1
+            col = getattr(node, "col_offset", None) or 0
+            context_line = self.source_lines[lineno - 1] if self.source_lines and 1 <= lineno <= len(self.source_lines) else ""
+            raise WrappedException(
+                "Error line %d, col %d:\n%s\nDescription: %s" % (lineno, col, context_line, str(e)), e, lineno, col, context_line
+            ) from e
+        except (ReturnException, BreakException, ContinueException) as e:
+            self.env_stack[0]["logger"].debug(f"Caught control flow exception: {type(e).__name__}, value: {getattr(e, 'value', None)}")
             self.recursion_depth -= 1
             raise
         except Exception as e:
+            self.env_stack[0]["logger"].debug(f"Caught unexpected exception: {type(e).__name__}, message: {str(e)}")
             self.recursion_depth -= 1
             if not wrap_exceptions:
                 raise
@@ -438,16 +468,26 @@ class ASTInterpreter:
         self.env_stack[0]['logger'].debug("Starting execute_async visit")
         self.env_stack[0]['logger'].debug(f"Parsed AST nodes: {[type(n).__name__ for n in ast.walk(node)]}")
         if entry_point:
-            await self.visit(node)  # Ensure all top-level definitions are processed
+            # Process definitions and call entry-point
+            await self.visit(node)
             func = self.get_variable(entry_point)
-            if func:
-                result = await func()
-            else:
+            if not func:
                 raise ValueError(f"Entry point '{entry_point}' not defined.")
+            self.env_stack[0]['logger'].debug(f"Debug: Calling entry point function {entry_point} of type {type(func)}")
+            try:
+                result = await func()
+            except StopAsyncIterationWithValue as e:
+                # Async generator return
+                result = e.value
+            except StopIteration as e:
+                # Sync generator or return in try/except
+                result = getattr(e, 'value', None)
+            self.env_stack[0]['logger'].debug(f"Debug: Result from function call: {result}, type: {type(result).__name__}")
         else:
             result = await self.visit(node)
+        self.env_stack[0]['logger'].debug(f"Final result in execute_async: {result}, type: {type(result).__name__}")
         local_vars = {k: v for k, v in self.env_stack[-1].items() if not k.startswith('__')}
-        self.env_stack[0]['logger'].debug(f"Returning from execute_async: result type {type(result).__name__}, value {result}")
+        self.env_stack[0]['logger'].debug(f"Returning from execute_async with result type {type(result).__name__}, value {result}")
         return (result, local_vars)
 
     def find_function_node(self, node: ast.Module, func_name: str) -> Optional[ast.AsyncFunctionDef]:
@@ -546,11 +586,9 @@ class ASTInterpreter:
                     except Exception as type_err:
                         self.env_stack[0]['logger'].debug(f"Error resolving exception type: {type_err}")
                         continue  # Skip this handler if type resolution fails
-                    self.env_stack[0]['logger'].debug("Checking handler for exception type: " + str(exc_type) + ", caught exception type: " + str(type(e).__name__))
-                    self.env_stack[0]['logger'].debug(f"Caught exception type: {type(e).__name__}, handler exception type: {exc_type}")
+                    self.env_stack[0]['logger'].debug(f"Resolved exception type for handler: {type(exc_type).__name__}, value: {exc_type}")
+                    self.env_stack[0]['logger'].debug(f"Caught exception: {type(e).__name__}, value: {e}")
                     self.env_stack[0]['logger'].debug(f"Checking if exception {e} is instance of {exc_type} or its cause if wrapped")
-                    self.env_stack[0]['logger'].debug(f"Resolved exc_type: {exc_type}, type of exc_type: {type(exc_type)}")
-                    self.env_stack[0]['logger'].debug(f"Caught exception: {e}, type of caught exception: {type(e)}")
                     if isinstance(e, exc_type) or (isinstance(e, WrappedException) and isinstance(e.__cause__, exc_type)):
                         self.env_stack[0]['logger'].debug("Exception matched, entering except block for handler at line " + str(getattr(handler, 'lineno', 'unknown')))
                         if handler.name:
@@ -560,6 +598,7 @@ class ASTInterpreter:
                             self.env_stack[0]['logger'].debug("Executed statement in except block: " + str(stmt.__class__.__name__) + ", result: " + str(stmt_result))
                             if isinstance(stmt, ast.Return):
                                 value = await self.visit(stmt.value, wrap_exceptions=wrap_exceptions) if stmt.value else None
+                                self.env_stack[0]['logger'].debug(f"Raising ReturnException in except block with value: {value}, type: {type(value).__name__ if value is not None else 'NoneType'}")
                                 raise ReturnException(value)
                         matched = True
                         break
@@ -572,6 +611,7 @@ class ASTInterpreter:
                         self.env_stack[0]['logger'].debug("Executed statement in bare except block: " + str(stmt.__class__.__name__) + ", result: " + str(stmt_result))
                         if isinstance(stmt, ast.Return):
                             value = await self.visit(stmt.value, wrap_exceptions=wrap_exceptions) if stmt.value else None
+                            self.env_stack[0]['logger'].debug(f"Raising ReturnException in bare except block with value: {value}, type: {type(value).__name__ if value is not None else 'NoneType'}")
                             raise ReturnException(value)
                     matched = True
                     break
@@ -586,6 +626,7 @@ class ASTInterpreter:
                     self.env_stack[0]['logger'].debug("Executed statement in orelse block: " + str(stmt.__class__.__name__) + ", result: " + str(stmt_result))
                     if isinstance(stmt, ast.Return):
                         value = await self.visit(stmt.value, wrap_exceptions=wrap_exceptions) if stmt.value else None
+                        self.env_stack[0]['logger'].debug(f"Raising ReturnException in orelse block with value: {value}, type: {type(value).__name__ if value is not None else 'NoneType'}")
                         raise ReturnException(value)
         finally:
             self.env_stack[0]['logger'].debug("Executing finally block if present")
@@ -595,13 +636,11 @@ class ASTInterpreter:
                     self.env_stack[0]['logger'].debug("Executed statement in finally block: " + str(stmt.__class__.__name__) + ", result: " + str(stmt_result))
                     if isinstance(stmt, ast.Return):
                         value = await self.visit(stmt.value, wrap_exceptions=wrap_exceptions) if stmt.value else None
+                        self.env_stack[0]['logger'].debug(f"Raising ReturnException in finally block with value: {value}, type: {type(value).__name__ if value is not None else 'NoneType'}")
                         raise ReturnException(value)
         self.env_stack[0]['logger'].debug("Exiting visit_Try")
         return None
 
-    def new_scope(self):
-        return Scope(self.env_stack)
-
     async def _resolve_exception_type(self, node: Optional[ast.AST]) -> Any:
         if node is None:
             return Exception
@@ -635,10 +674,13 @@ class ASTInterpreter:
         value = await self.visit(node.value, wrap_exceptions=True)
         try:
             result = await value
+        except StopAsyncIterationWithValue:
+            # Preserve return value of async generator
+            raise
+        except StopAsyncIteration:
+            # Standard generator exhaustion
+            raise
         except Exception as e:
-            # Propagate StopAsyncIteration without wrapping to allow async generator return value handling
-            if isinstance(e, StopAsyncIteration):
-                raise
             if wrap_exceptions:
                 raise RuntimeError(f"Error awaiting expression: {str(e)}") from e
             else:
