@@ -1,15 +1,14 @@
+# Dynamically import visitor modules and set as module attributes to avoid circular imports
 from .base_visitors import (
     visit_Module,
     visit_Expr,
     visit_Pass,
     visit_TypeIgnore,
 )
-
 from .import_visitors import (
     visit_Import,
     visit_ImportFrom,
 )
-
 from .literal_visitors import (
     visit_Constant,
     visit_Name,
@@ -25,21 +24,18 @@ from .literal_visitors import (
     visit_JoinedStr,
     visit_FormattedValue,
 )
-
 from .operator_visitors import (
     visit_BinOp,
     visit_UnaryOp,
     visit_Compare,
     visit_BoolOp,
 )
-
 from .assignment_visitors import (
     visit_Assign,
     visit_AugAssign,
     visit_AnnAssign,
     visit_NamedExpr,
 )
-
 from .control_flow_visitors import (
     visit_If,
     visit_While,
@@ -50,7 +46,6 @@ from .control_flow_visitors import (
     visit_Return,
     visit_IfExp,
 )
-
 from .function_visitors import (
     visit_FunctionDef,
     visit_AsyncFunctionDef,
@@ -58,29 +53,24 @@ from .function_visitors import (
     visit_Await,
     visit_Lambda,
 )
-
 from .comprehension_visitors import (
     visit_ListComp,
     visit_DictComp,
     visit_SetComp,
     visit_GeneratorExp,
 )
-
 from .exception_visitors import (
     visit_Try,
     visit_TryStar,
     visit_Raise,
 )
-
 from .class_visitors import (
     visit_ClassDef,
 )
-
 from .context_visitors import (
     visit_With,
     visit_AsyncWith,
 )
-
 from .misc_visitors import (
     visit_Global,
     visit_Nonlocal,
@@ -93,16 +83,62 @@ from .misc_visitors import (
 )
 
 __all__ = [
-    "visit_Import", "visit_ImportFrom", "visit_ListComp", "visit_Module", "visit_Expr",
-    "visit_Constant", "visit_Name", "visit_BinOp", "visit_UnaryOp", "visit_Assign",
-    "visit_AugAssign", "visit_AnnAssign", "visit_Compare", "visit_BoolOp", "visit_If",
-    "visit_While", "visit_For", "visit_Break", "visit_Continue", "visit_FunctionDef",
-    "visit_AsyncFunctionDef", "visit_Call", "visit_Await", "visit_Return", "visit_Lambda",
-    "visit_List", "visit_Tuple", "visit_Dict", "visit_Set", "visit_Attribute",
-    "visit_Subscript", "visit_Slice", "visit_Index", "visit_Starred", "visit_Pass",
-    "visit_TypeIgnore", "visit_Try", "visit_TryStar", "visit_Nonlocal", "visit_JoinedStr",
-    "visit_FormattedValue", "visit_GeneratorExp", "visit_ClassDef", "visit_With",
-    "visit_AsyncWith", "visit_Raise", "visit_Global", "visit_IfExp", "visit_DictComp",
-    "visit_SetComp", "visit_Yield", "visit_YieldFrom", "visit_Match", "visit_Delete",
-    "visit_AsyncFor", "visit_Assert", "visit_NamedExpr", "_match_pattern",
+    "visit_Import",
+    "visit_ImportFrom",
+    "visit_ListComp",
+    "visit_Module",
+    "visit_Expr",
+    "visit_Constant",
+    "visit_Name",
+    "visit_BinOp",
+    "visit_UnaryOp",
+    "visit_Assign",
+    "visit_AugAssign",
+    "visit_AnnAssign",
+    "visit_Compare",
+    "visit_BoolOp",
+    "visit_If",
+    "visit_While",
+    "visit_For",
+    "visit_Break",
+    "visit_Continue",
+    "visit_FunctionDef",
+    "visit_AsyncFunctionDef",
+    "visit_Call",
+    "visit_Await",
+    "visit_Return",
+    "visit_Lambda",
+    "visit_List",
+    "visit_Tuple",
+    "visit_Dict",
+    "visit_Set",
+    "visit_Attribute",
+    "visit_Subscript",
+    "visit_Slice",
+    "visit_Index",
+    "visit_Starred",
+    "visit_Pass",
+    "visit_TypeIgnore",
+    "visit_Try",
+    "visit_TryStar",
+    "visit_Nonlocal",
+    "visit_JoinedStr",
+    "visit_FormattedValue",
+    "visit_GeneratorExp",
+    "visit_ClassDef",
+    "visit_With",
+    "visit_AsyncWith",
+    "visit_Raise",
+    "visit_Global",
+    "visit_IfExp",
+    "visit_DictComp",
+    "visit_SetComp",
+    "visit_Yield",
+    "visit_YieldFrom",
+    "visit_Match",
+    "visit_Delete",
+    "visit_AsyncFor",
+    "visit_Assert",
+    "visit_NamedExpr",
+    "_match_pattern",
 ]
\ No newline at end of file
